@@ -33,6 +33,9 @@
 #include "ephy-history-service.h"
 #include "ephy-profile-utils.h"
 #include "ephy-snapshot-service.h"
+#ifdef HAVE_WEBKIT2
+#include "ephy-web-extension.h"
+#endif
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -55,6 +58,9 @@ struct _EphyEmbedShellPrivate
   EphyEmbedShellMode mode;
   EphyFrecentStore *frecent_store;
   guint single_initialised : 1;
+#ifdef HAVE_WEBKIT2
+  GDBusProxy *web_extension;
+#endif
 };
 
 enum
@@ -62,6 +68,7 @@ enum
   DOWNLOAD_ADDED,
   DOWNLOAD_REMOVED,
   PREPARE_CLOSE,
+  RESTORED_WINDOW,
   LAST_SIGNAL
 };
 
@@ -92,6 +99,9 @@ ephy_embed_shell_dispose (GObject *object)
   g_clear_object (&priv->global_history_service);
   g_clear_object (&priv->embed_single);
   g_clear_object (&priv->adblock_manager);
+#ifdef HAVE_WEBKIT2
+  g_clear_object (&priv->web_extension);
+#endif
 
   G_OBJECT_CLASS (ephy_embed_shell_parent_class)->dispose (object);
 }
@@ -263,6 +273,12 @@ ephy_embed_shell_prepare_close (EphyEmbedShell *shell)
   g_signal_emit (shell, signals[PREPARE_CLOSE], 0);
 }
 
+void
+ephy_embed_shell_restored_window (EphyEmbedShell *shell)
+{
+  g_signal_emit (shell, signals[RESTORED_WINDOW], 0);
+}
+
 static void
 ephy_embed_shell_set_property (GObject *object,
                                guint prop_id,
@@ -385,6 +401,23 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
+/**
+ * EphyEmbedShell::finished-restoring-window:
+ * @shell: the #EphyEmbedShell
+ *
+ * The ::finished-restoring-window signal is emitted when the
+ * session finishes restoring a window.
+ **/
+  signals[RESTORED_WINDOW] =
+    g_signal_new ("window-restored",
+                  EPHY_TYPE_EMBED_SHELL,
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (EphyEmbedShellClass, restored_window),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE,
+                  0);
+
   g_type_class_add_private (object_class, sizeof (EphyEmbedShellPrivate));
 }
 
@@ -653,3 +686,31 @@ ephy_embed_shell_launch_handler (EphyEmbedShell *shell,
 
   return ret;
 }
+
+#ifdef HAVE_WEBKIT2
+GDBusProxy *
+ephy_embed_shell_get_web_extension_proxy (EphyEmbedShell *shell)
+{
+  EphyEmbedShellPrivate *priv;
+
+  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
+
+  priv = shell->priv;
+  if (!priv->web_extension) {
+    char *service_name;
+
+    service_name = g_strdup_printf ("%s-%u", EPHY_WEB_EXTENSION_SERVICE_NAME, getpid ());
+    priv->web_extension = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                                         G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
+                                                         NULL,
+                                                         service_name,
+                                                         EPHY_WEB_EXTENSION_OBJECT_PATH,
+                                                         EPHY_WEB_EXTENSION_INTERFACE,
+                                                         NULL,
+                                                         NULL);
+    g_free (service_name);
+  }
+
+  return priv->web_extension;
+}
+#endif
