@@ -27,7 +27,6 @@
 #include "ephy-embed-shell.h"
 #include "ephy-embed-type-builtins.h"
 #include "ephy-file-helpers.h"
-#include "ephy-permission-manager.h"
 #include "ephy-prefs.h"
 #include "ephy-profile-utils.h"
 #include "ephy-request-about.h"
@@ -54,11 +53,7 @@ struct _EphyEmbedSinglePrivate {
 #endif
 };
 
-static void ephy_permission_manager_iface_init (EphyPermissionManagerIface *iface);
-
-G_DEFINE_TYPE_WITH_CODE (EphyEmbedSingle, ephy_embed_single, G_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (EPHY_TYPE_PERMISSION_MANAGER,
-                                                ephy_permission_manager_iface_init))
+G_DEFINE_TYPE (EphyEmbedSingle, ephy_embed_single, G_TYPE_OBJECT)
 
 static void
 form_auth_data_free (EphyEmbedSingleFormAuthData *data)
@@ -230,102 +225,9 @@ ephy_embed_single_class_init (EphyEmbedSingleClass *klass)
   object_class->finalize = ephy_embed_single_finalize;
   object_class->dispose = ephy_embed_single_dispose;
 
-  /**
-   * EphyEmbedSingle::new-window:
-   * @parent_embed: the #EphyEmbed requesting the new window, or %NULL
-   * @mask: a #EphyEmbedChrome
-   *
-   * The ::new_window signal is emitted when a new window needs to be opened.
-   * For example, when a JavaScript popup window was opened.
-   *
-   * Returns: (transfer none): a new #EphyEmbed.
-   **/
-  g_signal_new ("new-window",
-                EPHY_TYPE_EMBED_SINGLE,
-                G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
-                G_STRUCT_OFFSET (EphyEmbedSingleClass, new_window),
-                ephy_signal_accumulator_object, ephy_embed_get_type,
-                g_cclosure_marshal_generic,
-                GTK_TYPE_WIDGET,
-                2,
-                GTK_TYPE_WIDGET,
-                EPHY_TYPE_WEB_VIEW_CHROME);
-
-  /**
-   * EphyEmbedSingle::handle_content:
-   * @single:
-   * @mime_type: the MIME type of the content
-   * @address: the URL to the content
-   *
-   * The ::handle_content signal is emitted when encountering content of a mime
-   * type Epiphany is unable to handle itself.
-   *
-   * If a connected callback returns %TRUE, the signal will stop propagating. For
-   * example, this could be used by a download manager to prevent other
-   * ::handle_content listeners from being called.
-   **/
-  g_signal_new ("handle_content",
-                EPHY_TYPE_EMBED_SINGLE,
-                G_SIGNAL_RUN_LAST,
-                G_STRUCT_OFFSET (EphyEmbedSingleClass, handle_content),
-                g_signal_accumulator_true_handled, NULL,
-                g_cclosure_marshal_generic,
-                G_TYPE_BOOLEAN,
-                2,
-                G_TYPE_STRING,
-                G_TYPE_STRING);
-
   g_type_class_add_private (object_class, sizeof (EphyEmbedSinglePrivate));
 }
 
-static void
-impl_permission_manager_add (EphyPermissionManager *manager,
-                             const char *host,
-                             const char *type,
-                             EphyPermission permission)
-{
-}
-
-static void
-impl_permission_manager_remove (EphyPermissionManager *manager,
-                                const char *host,
-                                const char *type)
-{
-}
-
-static void
-impl_permission_manager_clear (EphyPermissionManager *manager)
-{
-}
-
-static EphyPermission
-impl_permission_manager_test (EphyPermissionManager *manager,
-                              const char *host,
-                              const char *type)
-{
-  g_return_val_if_fail (type != NULL && type[0] != '\0', EPHY_PERMISSION_DEFAULT);
-
-  return (EphyPermission)0;
-}
-
-static GList *
-impl_permission_manager_list (EphyPermissionManager *manager,
-                              const char *type)
-{
-  GList *list = NULL;
-  return list;
-}
-
-static void
-ephy_permission_manager_iface_init (EphyPermissionManagerIface *iface)
-{
-  iface->add = impl_permission_manager_add;
-  iface->remove = impl_permission_manager_remove;
-  iface->clear = impl_permission_manager_clear;
-  iface->test = impl_permission_manager_test;
-  iface->list = impl_permission_manager_list;
-}
-
 #ifndef HAVE_WEBKIT2
 static void
 cache_size_cb (GSettings *settings,
@@ -469,7 +371,7 @@ ephy_embed_single_initialize (EphyEmbedSingle *single)
   mode = ephy_embed_shell_get_mode (ephy_embed_shell_get_default ());
 
   /* WebKitSoupCache */
-  cache_dir = g_build_filename (mode == EPHY_EMBED_SHELL_MODE_PRIVATE ?
+  cache_dir = g_build_filename (EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (mode) ?
                                 ephy_dot_dir () : g_get_user_cache_dir (),
                                 g_get_prgname (), NULL);
   priv->cache = soup_cache_new (cache_dir, SOUP_CACHE_SINGLE_USER);
@@ -512,53 +414,6 @@ ephy_embed_single_clear_cache (EphyEmbedSingle *single)
 #endif
 }
 
-/**
- * ephy_embed_single_clear_auth_cache:
- * @single: the #EphyEmbedSingle
- * 
- * Clears the HTTP authentication cache.
- *
- * This does not clear regular website passwords; it only clears the HTTP
- * authentication cache. Websites which use HTTP authentication require the
- * browser to send a password along with every HTTP request; the browser will
- * ask the user for the password once and then cache the password for subsequent
- * HTTP requests. This function will clear the HTTP authentication cache,
- * meaning the user will have to re-enter a username and password the next time
- * Epiphany requests a web page secured with HTTP authentication.
- **/
-void
-ephy_embed_single_clear_auth_cache (EphyEmbedSingle *single)
-{
-}
-
-/**
- * ephy_embed_single_open_window:
- * @single: the #EphyEmbedSingle
- * @parent: the requested window's parent #EphyEmbed
- * @address: the URL to load
- * @name: a name for the window
- * @features: a Javascript features string
- *
- * Opens a new window, as if it were opened in @parent using the Javascript
- * method and arguments: <code>window.open(&quot;@address&quot;,
- * &quot;_blank&quot;, &quot;@features&quot;);</code>.
- * 
- * Returns: (transfer none): the new embed. This is either a #EphyEmbed, or,
- * when @features specified "chrome", a #GtkMozEmbed.
- *
- * NOTE: Use ephy_shell_new_tab() unless this handling of the @features string
- * is required.
- */
-GtkWidget *
-ephy_embed_single_open_window (EphyEmbedSingle *single,
-                               EphyEmbed *parent,
-                               const char *address,
-                               const char *name,
-                               const char *features)
-{
-  return NULL;
-}
-
 /**
  * ephy_embed_single_get_form_auth:
  * @single: an #EphyEmbedSingle
