@@ -43,13 +43,15 @@
 #include <webkit/webkit.h>
 #endif
 
-static void     ephy_embed_constructed      (GObject *object);
+static void     ephy_embed_constructed         (GObject *object);
 #ifndef HAVE_WEBKIT2
-static gboolean ephy_embed_inspect_show_cb  (WebKitWebInspector *inspector,
-                                             EphyEmbed *embed);
-static gboolean ephy_embed_inspect_close_cb (WebKitWebInspector *inspector,
-                                             EphyEmbed *embed);
+static gboolean ephy_embed_inspect_show_cb     (WebKitWebInspector *inspector,
+                                                EphyEmbed *embed);
+static gboolean ephy_embed_inspect_close_cb    (WebKitWebInspector *inspector,
+                                                EphyEmbed *embed);
 #endif
+static void     ephy_embed_restored_window_cb  (EphyEmbedShell *shell,
+                                                EphyEmbed *embed);
 
 #define EPHY_EMBED_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED, EphyEmbedPrivate))
 
@@ -79,6 +81,12 @@ struct _EphyEmbedPrivate
   GtkWidget *fullscreen_message_label;
   char *fullscreen_string;
 
+#ifdef HAVE_WEBKIT2
+  WebKitURIRequest *delayed_request;
+#else
+  WebKitNetworkRequest *delayed_request;
+#endif
+
   GtkWidget *overview;
   guint overview_mode : 1;
   GSList *messages;
@@ -290,12 +298,19 @@ load_changed_cb (WebKitWebView *web_view,
                  WebKitLoadEvent load_event,
                  EphyEmbed *embed)
 {
-  const char *address;
+  switch (load_event) {
+  case WEBKIT_LOAD_STARTED: {
+    const char *uri;
 
-  if (load_event == WEBKIT_LOAD_COMMITTED) {
+    uri = webkit_web_view_get_uri (web_view);
+    ephy_embed_set_overview_mode (embed, g_strcmp0 (uri, "ephy-about:overview") == 0);
+    break;
+  }
+  case WEBKIT_LOAD_COMMITTED:
     ephy_embed_destroy_top_widgets (embed);
-    address = ephy_web_view_get_address (EPHY_WEB_VIEW (web_view));
-    ephy_embed_set_overview_mode (embed, strcmp (address, "ephy-about:overview") == 0);
+    break;
+  default:
+    break;
   }
 }
 #else
@@ -415,6 +430,8 @@ ephy_embed_dispose (GObject *object)
     priv->adblock_handler_id = 0;
   }
 
+  g_clear_object (&priv->delayed_request);
+
   G_OBJECT_CLASS (ephy_embed_parent_class)->dispose (object);
 }
 
@@ -423,8 +440,11 @@ ephy_embed_finalize (GObject *object)
 {
   EphyEmbed *embed = EPHY_EMBED (object);
   EphyEmbedPrivate *priv = embed->priv;
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
   GSList *list;
 
+  g_signal_handlers_disconnect_by_func(shell, ephy_embed_restored_window_cb, embed);
+
   list = priv->destroy_on_transition_list;
   for (; list; list = list->next) {
     GtkWidget *widget = GTK_WIDGET (list->data);
@@ -559,14 +579,8 @@ static gboolean
 ephy_embed_attach_inspector_cb (WebKitWebInspector *inspector,
                                 EphyEmbed *embed)
 {
-  GtkAllocation allocation;
-  gtk_widget_get_allocation (GTK_WIDGET (embed->priv->scrolled_window), &allocation);
-
   embed->priv->inspector_attached = TRUE;
 
-  /* Set a sane position for the mover */
-  gtk_paned_set_position (embed->priv->paned, allocation.height * 0.5);
-
   gtk_widget_hide (embed->priv->inspector_window);
   gtk_widget_reparent (GTK_WIDGET (embed->priv->inspector_scrolled_window),
                        GTK_WIDGET (embed->priv->paned));
@@ -601,8 +615,14 @@ ephy_embed_inspect_show_cb (WebKitWebInspector *inspector,
   if (!embed->priv->inspector_attached) {
     gtk_widget_show_all (embed->priv->inspector_window);
     gtk_window_present (GTK_WINDOW (embed->priv->inspector_window));
-  } else
+  } else {
+    GtkAllocation allocation;
+    gtk_widget_get_allocation (GTK_WIDGET (embed->priv->scrolled_window), &allocation);
+
+    /* Set a sane position for the mover */
+    gtk_paned_set_position (embed->priv->paned, allocation.height * 0.5);
     gtk_widget_show (embed->priv->inspector_scrolled_window);
+  }
 
   return TRUE;
 }
@@ -861,11 +881,51 @@ setup_adblock (GSettings *settings,
 }
 #endif
 
+static void
+ephy_embed_maybe_load_delayed_request (EphyEmbed *embed)
+{
+  EphyEmbedPrivate *priv = embed->priv;
+  EphyWebView *web_view;
+
+  if (!priv->delayed_request)
+    return;
+
+  web_view = ephy_embed_get_web_view (embed);
+
+  ephy_web_view_load_request (web_view, priv->delayed_request);
+  g_clear_object (&priv->delayed_request);
+
+  /* This is to allow UI elements watching load status to show that the page is
+   * loading as soon as possible.
+   */
+#ifdef HAVE_WEBKIT2
+  g_signal_emit_by_name (web_view, "load-changed", WEBKIT_LOAD_STARTED);
+#else
+  g_object_notify (G_OBJECT (web_view), "load-status");
+#endif
+}
+
+static void
+ephy_embed_restored_window_cb (EphyEmbedShell *shell, EphyEmbed *embed)
+{
+  if (!gtk_widget_get_mapped (GTK_WIDGET (embed)))
+    return;
+
+  ephy_embed_maybe_load_delayed_request (embed);
+}
+
+static void
+ephy_embed_mapped_cb (GtkWidget *widget, gpointer data)
+{
+  ephy_embed_maybe_load_delayed_request ((EphyEmbed*)widget);
+}
+
 static void
 ephy_embed_constructed (GObject *object)
 {
   EphyEmbed *embed = (EphyEmbed*)object;
   EphyEmbedPrivate *priv = embed->priv;
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 #ifndef HAVE_WEBKIT2
   GtkWidget *scrolled_window;
 #endif
@@ -879,6 +939,12 @@ ephy_embed_constructed (GObject *object)
   WebKitWebInspector *inspector;
   GtkWidget *overlay;
 
+  g_signal_connect (shell, "window-restored",
+                    G_CALLBACK (ephy_embed_restored_window_cb), embed);
+
+  g_signal_connect (embed, "map",
+                    G_CALLBACK (ephy_embed_mapped_cb), NULL);
+
   /* Skeleton */
   web_view = WEBKIT_WEB_VIEW (ephy_web_view_new ());
 #ifndef HAVE_WEBKIT2
@@ -1006,13 +1072,14 @@ ephy_embed_constructed (GObject *object)
                     G_CALLBACK (ephy_embed_attach_inspector_cb),
                     embed);
 #else
-  priv->inspector_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  priv->inspector_attached = TRUE;
   priv->inspector_scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->inspector_scrolled_window),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_container_add (GTK_CONTAINER (priv->inspector_window),
+  gtk_container_add (GTK_CONTAINER (priv->paned),
                      priv->inspector_scrolled_window);
 
+  priv->inspector_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title (GTK_WINDOW (priv->inspector_window),
                         _("Web Inspector"));
   gtk_window_set_default_size (GTK_WINDOW (priv->inspector_window),
@@ -1159,6 +1226,51 @@ ephy_embed_get_overview_mode (EphyEmbed *embed)
   return embed->priv->overview_mode;
 }
 
+/**
+ * ephy_embed_set_delayed_load_request:
+ * @embed: a #EphyEmbed
+ * @request: a #WebKitNetworkRequest
+ *
+ * Sets the #WebKitNetworkRequest that should be loaded when the tab this embed
+ * is on is switched to.
+ */
+#ifdef HAVE_WEBKIT2
+void
+ephy_embed_set_delayed_load_request (EphyEmbed *embed, WebKitURIRequest *request)
+#else
+void
+ephy_embed_set_delayed_load_request (EphyEmbed *embed, WebKitNetworkRequest *request)
+#endif
+{
+  g_return_if_fail (EPHY_IS_EMBED (embed));
+#ifdef HAVE_WEBKIT2
+  g_return_if_fail (WEBKIT_IS_URI_REQUEST (request));
+#else
+  g_return_if_fail (WEBKIT_IS_NETWORK_REQUEST (request));
+#endif
+
+  g_clear_object (&embed->priv->delayed_request);
+
+  g_object_ref (request);
+  embed->priv->delayed_request = request;
+}
+
+/**
+ * ephy_embed_has_load_pending:
+ * @embed: a #EphyEmbed
+ *
+ * Checks whether a load has been delayed for this #EphyEmbed.
+ *
+ * Returns: %TRUE or %FALSE
+ */
+gboolean
+ephy_embed_has_load_pending (EphyEmbed *embed)
+{
+  g_return_val_if_fail (EPHY_IS_EMBED (embed), FALSE);
+
+  return !!embed->priv->delayed_request;
+}
+
 /**
  * ephy_embed_get_overview:
  * @embed: a #EphyEmbed
