@@ -229,6 +229,9 @@ main (int argc,
   EphyShell *ephy_shell;
   int status;
   EphyFileHelpersFlags flags;
+#ifdef HAVE_WEBKIT2
+  char *pid_str;
+#endif
 
 #ifdef ENABLE_NLS
   /* Initialize the i18n stuff */
@@ -419,8 +422,7 @@ main (int argc,
   if (private_instance)
     mode = EPHY_EMBED_SHELL_MODE_PRIVATE;
   else if (incognito_mode) {
-    /* Internally incognito mode is also a private mode. */
-    mode = EPHY_EMBED_SHELL_MODE_PRIVATE;
+    mode = EPHY_EMBED_SHELL_MODE_INCOGNITO;
 
     /* Use the right theming. */
     g_object_set (gtk_settings_get_default (),
@@ -459,6 +461,16 @@ main (int argc,
     gtk_window_set_default_icon_name ("web-browser");
   }
 
+#ifdef HAVE_WEBKIT2
+  pid_str = g_strdup_printf ("%u", getpid ());
+  g_setenv ("EPHY_WEB_EXTENSION_ID", pid_str, TRUE);
+  g_free (pid_str);
+
+  /* Set the web extensions dir ASAP before the process is launched */
+  webkit_web_context_set_web_extensions_directory (webkit_web_context_get_default (),
+                                                   EPHY_WEB_EXTENSIONS_DIR);
+#endif
+
   ephy_embed_prefs_init ();
   _ephy_shell_create_instance (mode);
 
