@@ -30,8 +30,11 @@
 #include "ephy-embed.h"
 #include "ephy-file-helpers.h"
 #include "ephy-gui.h"
+#include "ephy-notebook.h"
 #include "ephy-prefs.h"
+#include "ephy-private.h"
 #include "ephy-settings.h"
+#include "ephy-shell.h"
 #include "ephy-string.h"
 #include "ephy-window.h"
 
@@ -42,14 +45,29 @@
 
 #define EPHY_SESSION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_SESSION, EphySessionPrivate))
 
+typedef struct
+{
+	gpointer* parent_location;
+	int position;
+	char *url;
+	GList *bflist;
+} ClosedTab;
+
 struct _EphySessionPrivate
 {
-	guint open_uris_idle_id;
+	GQueue *closed_tabs;
 	GCancellable *save_cancellable;
 	guint dont_save : 1;
 };
 
 #define SESSION_STATE		"type:session_state"
+#define MAX_CLOSED_TABS		10
+
+enum
+{
+	PROP_0,
+	PROP_CAN_UNDO_TAB_CLOSED
+};
 
 G_DEFINE_TYPE (EphySession, ephy_session, G_TYPE_OBJECT)
 
@@ -121,6 +139,256 @@ load_status_notify_cb (EphyWebView *view,
 }
 #endif
 
+static gpointer *
+parent_location_new (EphyNotebook *notebook)
+{
+	gpointer *location = g_slice_new (gpointer);
+	*location = notebook;
+	g_object_add_weak_pointer (G_OBJECT (notebook), location);
+
+	return location;
+}
+
+static void
+parent_location_free (gpointer *location, gboolean last_reference)
+{
+	if (!location)
+		return;
+
+	if (*location && last_reference)
+	{
+		g_object_remove_weak_pointer (G_OBJECT (*location), location);
+	}
+
+	g_slice_free (gpointer, location);
+}
+
+static void
+closed_tab_free (ClosedTab *tab)
+{
+	if (tab->bflist)
+	{
+		g_list_free_full (tab->bflist, g_object_unref);
+		tab->bflist = NULL;
+	}
+
+	if (tab->url)
+	{
+		g_free (tab->url);
+		tab->url = NULL;
+	}
+
+	g_slice_free (ClosedTab, tab);
+}
+
+static int
+compare_func (ClosedTab *iter, EphyNotebook *notebook)
+{
+	return (EphyNotebook *)*iter->parent_location - notebook;
+}
+
+static ClosedTab *
+find_tab_with_notebook (GQueue *queue, EphyNotebook *notebook)
+{
+	GList *item = g_queue_find_custom (queue, notebook, (GCompareFunc)compare_func);
+	return item ? (ClosedTab*)item->data : NULL;
+}
+
+static ClosedTab *
+closed_tab_new (GQueue *closed_tabs,
+		const char *address,
+		GList *bflist,
+		int position,
+		EphyNotebook *parent_notebook)
+{
+	ClosedTab *tab = g_slice_new0 (ClosedTab);
+	ClosedTab *sibling_tab;
+
+	tab->url = g_strdup (address);
+#ifndef HAVE_WEBKIT2
+	tab->bflist = g_list_copy_deep (bflist, (GCopyFunc)webkit_web_history_item_copy, NULL);
+#endif
+	tab->position = position;
+
+	sibling_tab = find_tab_with_notebook (closed_tabs, parent_notebook);
+	if (sibling_tab)
+		tab->parent_location = sibling_tab->parent_location;
+	else
+		tab->parent_location = parent_location_new (parent_notebook);
+
+	return tab;
+}
+
+static void
+post_restore_cleanup (GQueue *closed_tabs, ClosedTab *restored_tab, gboolean notebook_is_new)
+{
+
+	if (find_tab_with_notebook (closed_tabs, *restored_tab->parent_location))
+	{
+		if (notebook_is_new == TRUE)
+		{
+			/* If this is a newly opened notebook and
+			   there are other tabs that must be restored
+			   here, add a weak poiner to keep track of
+			   the lifetime of it. */
+			g_object_add_weak_pointer (G_OBJECT (*restored_tab->parent_location),
+						   restored_tab->parent_location);
+		}
+	}
+	else
+	{
+		/* If there are no other tabs that must be restored to this notebook,
+		   we can remove the pointer keeping track of its location.
+		   If this is a new window, we don't need to remove any weak
+		   pointer, as no one has been added yet. */
+		parent_location_free (restored_tab->parent_location, !notebook_is_new);
+	}
+}
+
+void
+ephy_session_undo_close_tab (EphySession *session)
+{
+	EphySessionPrivate *priv;
+	EphyEmbed *embed, *new_tab;
+	ClosedTab *tab;
+#ifndef HAVE_WEBKIT2
+	WebKitWebBackForwardList *dest;
+	GList *i;
+#endif
+	EphyNewTabFlags flags = EPHY_NEW_TAB_OPEN_PAGE
+		| EPHY_NEW_TAB_PRESENT_WINDOW
+		| EPHY_NEW_TAB_JUMP
+		| EPHY_NEW_TAB_DONT_COPY_HISTORY;
+
+	g_return_if_fail (EPHY_IS_SESSION (session));
+
+	priv = session->priv;
+
+	tab = g_queue_pop_head (priv->closed_tabs);
+	if (tab == NULL)
+		return;
+
+	LOG ("UNDO CLOSE TAB: %s", tab->url);
+	if (*tab->parent_location != NULL)
+	{
+		GtkWidget *window;
+
+		flags |= EPHY_NEW_TAB_IN_EXISTING_WINDOW;
+
+		if (tab->position > 0)
+		{
+			/* Append in the n-th position. */
+			embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (*tab->parent_location),
+								       tab->position - 1));
+			flags |= EPHY_NEW_TAB_APPEND_AFTER;
+		}
+		else
+		{
+			/* Just prepend in the first position. */
+			embed = NULL;
+			flags |= EPHY_NEW_TAB_FIRST;
+		}
+
+		window = gtk_widget_get_toplevel (GTK_WIDGET (*tab->parent_location));
+		new_tab = ephy_shell_new_tab (ephy_shell_get_default (),
+					      EPHY_WINDOW (window), embed, tab->url,
+					      flags);
+		post_restore_cleanup (priv->closed_tabs, tab, FALSE);
+	}
+	else
+	{
+		EphyNotebook *notebook;
+		flags |=  EPHY_NEW_TAB_IN_NEW_WINDOW;
+		new_tab = ephy_shell_new_tab (ephy_shell_get_default (),
+					      NULL, NULL, tab->url, flags);
+
+		/* FIXME: This makes the assumption that the notebook
+		   is the parent of the returned EphyEmbed. */
+		notebook = EPHY_NOTEBOOK (gtk_widget_get_parent (GTK_WIDGET (new_tab)));
+		*tab->parent_location = notebook;
+		post_restore_cleanup (priv->closed_tabs, tab, TRUE);
+	}
+
+	/* This is deficient: we need to recreate the whole
+	 * BackForward list. Also, WebKit2 doesn't have this API. */
+#ifndef HAVE_WEBKIT2
+	dest = webkit_web_view_get_back_forward_list (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (new_tab));
+	for (i = tab->bflist; i; i = i->next)
+	{
+		LOG ("ADDING TO BF: %s",
+		     webkit_web_history_item_get_title ((WebKitWebHistoryItem*) i->data));
+		webkit_web_back_forward_list_add_item (dest,
+						       webkit_web_history_item_copy ((WebKitWebHistoryItem*) i->data));
+	}
+#endif
+	closed_tab_free (tab);
+
+	if (g_queue_is_empty (priv->closed_tabs))
+		g_object_notify (G_OBJECT (session), "can-undo-tab-closed");
+}
+
+static void
+ephy_session_tab_closed (EphySession *session,
+			 EphyNotebook *notebook,
+			 EphyEmbed *embed,
+			 gint position)
+{
+	EphySessionPrivate *priv = session->priv;
+	EphyWebView *view;
+	const char *address;
+#ifdef HAVE_WEBKIT2
+	WebKitBackForwardList *source;
+#else
+	WebKitWebBackForwardList *source;
+#endif
+	ClosedTab *tab;
+	GList *items = NULL;
+
+	view = ephy_embed_get_web_view (embed);
+	address = ephy_web_view_get_address (view);
+
+	source = webkit_web_view_get_back_forward_list (WEBKIT_WEB_VIEW (view));
+#ifdef HAVE_WEBKIT2
+	items = webkit_back_forward_list_get_back_list_with_limit (source, EPHY_WEBKIT_BACK_FORWARD_LIMIT);
+#else
+	items = webkit_web_back_forward_list_get_back_list_with_limit (source, EPHY_WEBKIT_BACK_FORWARD_LIMIT);
+#endif
+	if (items == NULL && g_strcmp0 (address, "ephy-about:overview") == 0)
+		return;
+
+	if (g_queue_get_length (priv->closed_tabs) == MAX_CLOSED_TABS)
+	{
+		tab = g_queue_pop_tail (priv->closed_tabs);
+		if (tab->parent_location && !find_tab_with_notebook (priv->closed_tabs, *tab->parent_location))
+		{
+			parent_location_free (tab->parent_location, TRUE);
+		}
+
+		closed_tab_free (tab);
+		tab = NULL;
+	}
+
+	items = g_list_reverse (items);
+	tab = closed_tab_new (priv->closed_tabs, address, items, position, notebook);
+	g_list_free (items);
+
+	g_queue_push_head (priv->closed_tabs, tab);
+
+	if (g_queue_get_length (priv->closed_tabs) == 1)
+		g_object_notify (G_OBJECT (session), "can-undo-tab-closed");
+
+	LOG ("Added: %s to the list (%d elements)",
+	     address, g_queue_get_length (priv->closed_tabs));
+}
+
+gboolean
+ephy_session_get_can_undo_tab_closed (EphySession *session)
+{
+	g_return_val_if_fail (EPHY_IS_SESSION (session), FALSE);
+
+	return g_queue_is_empty (session->priv->closed_tabs) == FALSE;
+}
+
 static void
 notebook_page_added_cb (GtkWidget *notebook,
 			EphyEmbed *embed,
@@ -153,6 +421,7 @@ notebook_page_removed_cb (GtkWidget *notebook,
 		(ephy_embed_get_web_view (embed), G_CALLBACK (load_status_notify_cb),
 		 session);
 #endif
+	ephy_session_tab_closed (session, EPHY_NOTEBOOK (notebook), embed, position);
 }
 
 static void
@@ -164,144 +433,6 @@ notebook_page_reordered_cb (GtkWidget *notebook,
 	ephy_session_save (session, SESSION_STATE);
 }
 
-typedef struct {
-	EphySession *session;
-	EphyWindow *window;
-	char **uris;
-	EphyNewTabFlags flags;
-	guint32 user_time;
-	guint current_uri;
-} OpenURIsData;
-
-static OpenURIsData *
-open_uris_data_new (EphySession *session,
-		    const char **uris,
-		    EphyStartupFlags startup_flags,
-		    guint32 user_time)
-{
-	OpenURIsData *data;
-	EphyShell *shell;
-	gboolean new_windows_in_tabs;
-	gboolean have_uris;
-
-	shell = ephy_shell_get_default ();
-
-	data = g_slice_new0 (OpenURIsData);
-	data->session = g_object_ref (session);
-	data->uris = g_strdupv ((char **)uris);
-	data->user_time = user_time;
-
-	data->window = ephy_shell_get_main_window (shell);
-
-	new_windows_in_tabs = g_settings_get_boolean (EPHY_SETTINGS_MAIN,
-						      EPHY_PREFS_NEW_WINDOWS_IN_TABS);
-
-	have_uris = ! (g_strv_length ((char **)uris) == 1 && g_str_equal (uris[0], ""));
-
-	if (startup_flags & EPHY_STARTUP_NEW_TAB)
-	{
-		data->flags |= EPHY_NEW_TAB_FROM_EXTERNAL;
-	}
-	if (startup_flags & EPHY_STARTUP_NEW_WINDOW)
-	{
-		data->window = NULL;
-		data->flags |= EPHY_NEW_TAB_IN_NEW_WINDOW;
-	}
-	else if (startup_flags & EPHY_STARTUP_NEW_TAB || (new_windows_in_tabs && have_uris))
-	{
-		data->flags |= EPHY_NEW_TAB_IN_EXISTING_WINDOW | EPHY_NEW_TAB_JUMP | EPHY_NEW_TAB_PRESENT_WINDOW;
-	}
-	else if (!have_uris)
-	{
-		data->window = NULL;
-		data->flags |= EPHY_NEW_TAB_IN_NEW_WINDOW;
-	}
-
-	g_application_hold (G_APPLICATION (shell));
-
-	return data;
-}
-
-static void
-open_uris_data_free (OpenURIsData *data)
-{
-	g_application_release (G_APPLICATION (ephy_shell_get_default ()));
-	g_object_unref (data->session);
-	g_strfreev (data->uris);
-
-	g_slice_free (OpenURIsData, data);
-}
-
-static gboolean
-ephy_session_open_uris_idle (OpenURIsData *data)
-{
-	EphyEmbed *embed;
-	EphyNewTabFlags page_flags;
-	const char *url;
-#ifdef HAVE_WEBKIT2
-	WebKitURIRequest *request = NULL;
-#else
-	WebKitNetworkRequest *request = NULL;
-#endif
-
-	url = data->uris[data->current_uri];
-	if (url[0] == '\0')
-	{
-		page_flags = EPHY_NEW_TAB_HOME_PAGE;
-	}
-	else
-	{
-		page_flags = EPHY_NEW_TAB_OPEN_PAGE;
-#ifdef HAVE_WEBKIT2
-		request = webkit_uri_request_new (url);
-#else
-		request = webkit_network_request_new (url);
-#endif
-	}
-
-	embed = ephy_shell_new_tab_full (ephy_shell_get_default (),
-					 data->window,
-					 NULL /* parent tab */,
-					 request,
-					 data->flags | page_flags,
-					 EPHY_WEB_VIEW_CHROME_ALL,
-					 FALSE /* is popup? */,
-					 data->user_time);
-
-	if (request)
-	{
-		g_object_unref (request);
-	}
-
-	data->window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed)));
-	data->current_uri++;
-
-	return data->uris[data->current_uri] != NULL;
-}
-
-static void
-ephy_session_open_uris_idle_done (OpenURIsData *data)
-{
-	data->session->priv->open_uris_idle_id = 0;
-	open_uris_data_free (data);
-}
-
-void
-ephy_session_open_uris (EphySession *session,
-			const char **uris,
-			EphyStartupFlags startup_flags,
-			guint32 user_time)
-{
-	if (session->priv->open_uris_idle_id == 0)
-	{
-		session->priv->open_uris_idle_id =
-			g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
-					 (GSourceFunc)ephy_session_open_uris_idle,
-					 open_uris_data_new (session, uris, startup_flags, user_time),
-					 (GDestroyNotify)ephy_session_open_uris_idle_done);
-	}
-}
-
 static void
 session_maybe_open_window (EphySession *session,
 			   guint32 user_time)
@@ -385,6 +516,7 @@ ephy_session_init (EphySession *session)
 
 	session->priv = EPHY_SESSION_GET_PRIVATE (session);
 
+	session->priv->closed_tabs = g_queue_new ();
 	shell = ephy_shell_get_default ();
 	g_signal_connect (shell, "window-added",
 			  G_CALLBACK (window_added_cb), session);
@@ -399,21 +531,46 @@ ephy_session_dispose (GObject *object)
 
 	LOG ("EphySession disposing");
 
-	if (session->priv->open_uris_idle_id > 0)
-	{
-		g_source_remove (session->priv->open_uris_idle_id);
-		session->priv->open_uris_idle_id = 0;
-	}
+	g_queue_free_full (session->priv->closed_tabs,
+			   (GDestroyNotify)closed_tab_free);
 
 	G_OBJECT_CLASS (ephy_session_parent_class)->dispose (object);
 }
 
+static void
+ephy_session_get_property (GObject        *object,
+			   guint           property_id,
+			   GValue         *value,
+			   GParamSpec     *pspec)
+{
+	EphySession *session = EPHY_SESSION (object);
+
+	switch (property_id)
+	{
+	case PROP_CAN_UNDO_TAB_CLOSED:
+		g_value_set_boolean (value,
+				     ephy_session_get_can_undo_tab_closed (session));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+	}
+}
+
 static void
 ephy_session_class_init (EphySessionClass *class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
 
 	object_class->dispose = ephy_session_dispose;
+	object_class->get_property = ephy_session_get_property;
+
+	g_object_class_install_property (object_class,
+					 PROP_CAN_UNDO_TAB_CLOSED,
+					 g_param_spec_boolean ("can-undo-tab-closed",
+							       "Can undo tab close",
+							       "Session can undo a tab closure",
+							       FALSE,
+							       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	g_type_class_add_private (object_class, sizeof (EphySessionPrivate));
 }
@@ -474,7 +631,7 @@ session_tab_new (EphyEmbed *embed)
 	}
 
 	session_tab->title = g_strdup (ephy_web_view_get_title (web_view));
-	session_tab->loading = ephy_web_view_is_loading (web_view);
+	session_tab->loading = ephy_web_view_is_loading (web_view) && !ephy_embed_has_load_pending (embed);
 
 	return session_tab;
 }
@@ -560,7 +717,7 @@ save_data_new (EphySession *session,
 	data->session = g_object_ref (session);
 	data->save_file = get_session_file (filename);
 
-	windows = ephy_shell_get_windows (shell);
+	windows = gtk_application_get_windows (GTK_APPLICATION (shell));
 	for (w = windows; w != NULL ; w = w->next)
 	{
 		SessionWindow *session_window;
@@ -569,7 +726,6 @@ save_data_new (EphySession *session,
 		if (session_window)
 			data->windows = g_list_prepend (data->windows, session_window);
 	}
-	g_list_free (windows);
 	data->windows = g_list_reverse (data->windows);
 
 	return data;
@@ -675,30 +831,28 @@ write_ephy_window (xmlTextWriterPtr writer,
 }
 
 static void
-session_save_finished (EphySession *session)
+save_session_in_thread_cb (GObject *source_object,
+			   GAsyncResult *res,
+			   gpointer user_data)
 {
 	g_application_release (G_APPLICATION (ephy_shell_get_default ()));
 }
 
-static gboolean
-save_session_in_thread (GIOSchedulerJob *job,
-			GCancellable *cancellable,
-			gpointer user_data)
+static void
+save_session_sync (GTask *task,
+		   gpointer source_object,
+		   gpointer task_data,
+		   GCancellable *cancellable)
 {
-	SaveData *data = (SaveData *)user_data;
+	SaveData *data = (SaveData *)g_task_get_task_data (task);
 	xmlBufferPtr buffer;
 	xmlTextWriterPtr writer;
 	GList *w;
-	int ret;
+	int ret = -1;
 
 	buffer = xmlBufferCreate ();
 	writer = xmlNewTextWriterMemory (buffer, 0);
-	if (writer == NULL)
-	{
-		xmlBufferFree (buffer);
-
-		return FALSE;
-	}
+	if (writer == NULL) goto out;
 
 	ret = xmlTextWriterSetIndent (writer, 1);
 	if (ret < 0) goto out;
@@ -728,7 +882,8 @@ save_session_in_thread (GIOSchedulerJob *job,
 	ret = xmlTextWriterEndDocument (writer);
 
 out:
-	xmlFreeTextWriter (writer);
+	if (writer)
+		xmlFreeTextWriter (writer);
 
 	if (ret >= 0 && !g_cancellable_is_cancelled (cancellable))
 	{
@@ -750,14 +905,9 @@ out:
 
 	xmlBufferFree (buffer);
 
-	g_io_scheduler_job_send_to_mainloop_async (job,
-						   (GSourceFunc) session_save_finished,
-						   g_object_ref (data->session),
-						   (GDestroyNotify) g_object_unref);
+	g_task_return_boolean (task, TRUE);
 
 	STOP_PROFILER ("Saving session")
-
-	return FALSE;
 }
 
 void
@@ -767,6 +917,7 @@ ephy_session_save (EphySession *session,
 	EphySessionPrivate *priv;
 	EphyShell *shell;
 	SaveData *data;
+	GTask *task;
 
 	g_return_if_fail (EPHY_IS_SESSION (session));
 
@@ -797,8 +948,11 @@ ephy_session_save (EphySession *session,
 	priv->save_cancellable = g_cancellable_new ();
 	data = save_data_new (session, filename);
 	g_application_hold (G_APPLICATION (shell));
-	g_io_scheduler_push_job (save_session_in_thread, data, (GDestroyNotify)save_data_free,
-				 G_PRIORITY_DEFAULT, priv->save_cancellable);
+
+	task = g_task_new (session, priv->save_cancellable,
+			   save_session_in_thread_cb, NULL);
+	g_task_set_task_data (task, data, (GDestroyNotify)save_data_free);
+	g_task_run_in_thread (task, save_session_sync);
 }
 
 static void
@@ -948,11 +1102,19 @@ session_parse_embed (SessionParserContext *context,
 	 */
 	if (!was_loading || is_blank_page)
 	{
-		ephy_shell_new_tab (ephy_shell_get_default (),
-				    context->window, NULL, url,
-				    EPHY_NEW_TAB_IN_EXISTING_WINDOW |
-				    EPHY_NEW_TAB_OPEN_PAGE |
-				    EPHY_NEW_TAB_APPEND_LAST);
+		EphyNewTabFlags flags;
+		EphyEmbed *embed;
+		EphyWebView *web_view;
+
+		flags = EPHY_NEW_TAB_IN_EXISTING_WINDOW;
+		flags |= EPHY_NEW_TAB_APPEND_LAST;
+	        flags |= EPHY_NEW_TAB_DELAYED_OPEN_PAGE;
+
+		embed = ephy_shell_new_tab (ephy_shell_get_default (),
+					    context->window, NULL, url, flags);
+
+		web_view = ephy_embed_get_web_view (embed);
+		ephy_web_view_set_placeholder (web_view, url, title);
 	}
 	else if (was_loading && url != NULL)
 	{
@@ -996,6 +1158,7 @@ session_end_element (GMarkupParseContext  *ctx,
 	if (strcmp (element_name, "window") == 0)
 	{
 		GtkWidget *notebook;
+		EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
 		notebook = ephy_window_get_notebook (context->window);
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), context->active_tab);
@@ -1009,6 +1172,8 @@ session_end_element (GMarkupParseContext  *ctx,
 			gtk_widget_show (GTK_WIDGET (context->window));
 		}
 
+		ephy_embed_shell_restored_window (shell);
+
 		context->window = NULL;
 		context->active_tab = 0;
 		context->is_first_window = FALSE;
@@ -1465,3 +1630,23 @@ ephy_session_resume_finish (EphySession *session,
 	return !g_simple_async_result_propagate_error (simple, error);
 }
 
+
+void
+ephy_session_clear (EphySession *session)
+{
+	EphyShell *shell;
+	GList *windows, *p;
+
+	g_return_if_fail (EPHY_IS_SESSION (session));
+
+	shell = ephy_shell_get_default ();
+	windows = g_list_copy (gtk_application_get_windows (GTK_APPLICATION (shell)));
+	for (p = windows; p; p = p->next)
+		gtk_widget_destroy (GTK_WIDGET (p->data));
+	g_list_free (windows);
+	g_queue_foreach (session->priv->closed_tabs,
+			 (GFunc)closed_tab_free, NULL);
+	g_queue_clear (session->priv->closed_tabs);
+
+	ephy_session_save (session, SESSION_STATE);
+}
