@@ -63,7 +63,7 @@ struct _EphyShellPrivate {
   GObject *prefs_dialog;
   GList *del_on_exit;
   EphyShellStartupContext *startup_context;
-  guint embed_single_connected : 1;
+  guint open_uris_idle_id;
 };
 
 EphyShell *ephy_shell = NULL;
@@ -72,7 +72,6 @@ static void ephy_shell_class_init (EphyShellClass *klass);
 static void ephy_shell_init   (EphyShell *shell);
 static void ephy_shell_dispose    (GObject *object);
 static void ephy_shell_finalize   (GObject *object);
-static GObject *impl_get_embed_single   (EphyEmbedShell *embed_shell);
 
 G_DEFINE_TYPE (EphyShell, ephy_shell, EPHY_TYPE_EMBED_SHELL)
 
@@ -118,7 +117,7 @@ ephy_shell_startup_continue (EphyShell *shell)
   EphyShellStartupContext *ctx;
   EphySession *session;
 
-  session = EPHY_SESSION (ephy_shell_get_session (shell));
+  session = ephy_shell_get_session (shell);
   g_assert (session != NULL);
 
   ctx = shell->priv->startup_context;
@@ -129,8 +128,8 @@ ephy_shell_startup_continue (EphyShell *shell)
   else if (ctx->arguments != NULL) {
     /* Don't queue any window openings if no extra arguments given, */
     /* since session autoresume will open one for us. */
-    ephy_session_open_uris (session, (const char **)ctx->arguments,
-                            ctx->startup_flags, ctx->user_time);
+    ephy_shell_open_uris (shell, (const char **)ctx->arguments,
+                          ctx->startup_flags, ctx->user_time);
   }
 }
 
@@ -150,6 +149,14 @@ new_incognito_window (GSimpleAction *action,
   window_cmd_file_new_incognito_window (NULL, NULL);
 }
 
+static void
+reopen_closed_tab (GSimpleAction *action,
+                   GVariant *parameter,
+                   gpointer user_data)
+{
+  window_cmd_undo_close_tab (NULL, NULL);
+}
+
 static void
 show_bookmarks (GSimpleAction *action,
                 GVariant *parameter,
@@ -213,6 +220,10 @@ static GActionEntry app_entries[] = {
   { "quit", quit_application, NULL, NULL, NULL },
 };
 
+static GActionEntry app_normal_mode_entries[] = {
+  { "reopen-closed-tab", reopen_closed_tab, NULL, NULL, NULL },
+};
+
 static void
 ephy_shell_startup (GApplication* application)
 {
@@ -230,6 +241,18 @@ ephy_shell_startup (GApplication* application)
                                      app_entries, G_N_ELEMENTS (app_entries),
                                      application);
 
+    if (mode != EPHY_EMBED_SHELL_MODE_INCOGNITO) {
+      g_action_map_add_action_entries (G_ACTION_MAP (application),
+                                       app_normal_mode_entries, G_N_ELEMENTS (app_normal_mode_entries),
+                                       application);
+      g_object_bind_property (G_OBJECT (ephy_shell_get_session (EPHY_SHELL (application))),
+                              "can-undo-tab-closed",
+                              g_action_map_lookup_action (G_ACTION_MAP (application),
+                                                          "reopen-closed-tab"),
+                              "enabled",
+                              G_BINDING_SYNC_CREATE);
+    }
+
     builder = gtk_builder_new ();
     gtk_builder_add_from_resource (builder,
                                    "/org/gnome/epiphany/epiphany-application-menu.ui",
@@ -266,7 +289,7 @@ ephy_shell_activate (GApplication *application)
     EphyShellStartupContext *ctx;
 
     ctx = shell->priv->startup_context;
-    ephy_session_resume (EPHY_SESSION (ephy_shell_get_session (shell)),
+    ephy_session_resume (ephy_shell_get_session (shell),
                          ctx->user_time, NULL, session_load_cb, shell);
   } else
     ephy_shell_startup_continue (shell);
@@ -430,50 +453,6 @@ ephy_shell_before_emit (GApplication *application,
                                                               platform_data);
 }
 
-static gboolean
-window_focus_in_event_cb (EphyWindow *window,
-                          GdkEventFocus *event,
-                          EphyShell *shell)
-{
-  LOG ("focus-in-event for window %p", window);
-
-  g_return_val_if_fail (g_list_find (shell->priv->windows, window) != NULL, FALSE);
-
-  /* move the active window to the front of the list */
-  shell->priv->windows = g_list_remove (shell->priv->windows, window);
-  shell->priv->windows = g_list_prepend (shell->priv->windows, window);
-
-  return GDK_EVENT_PROPAGATE;
-}
-
-static void
-ephy_shell_window_added (GtkApplication *application,
-                         GtkWindow *window)
-{
-  EphyShell *shell = EPHY_SHELL (application);
-
-  if (EPHY_IS_WINDOW (window)) {
-    shell->priv->windows = g_list_append (shell->priv->windows, window);
-    g_signal_connect (window, "focus-in-event",
-                      G_CALLBACK (window_focus_in_event_cb),
-                      shell);
-  }
-
-  GTK_APPLICATION_CLASS (ephy_shell_parent_class)->window_added (application, window);
-}
-
-static void
-ephy_shell_window_removed (GtkApplication *application,
-                           GtkWindow *window)
-{
-  EphyShell *shell = EPHY_SHELL (application);
-
-  if (EPHY_IS_WINDOW (window))
-    shell->priv->windows = g_list_remove (shell->priv->windows, window);
-
-  GTK_APPLICATION_CLASS (ephy_shell_parent_class)->window_removed (application, window);
-}
-
 static GObject *
 ephy_shell_get_lockdown (EphyShell *shell)
 {
@@ -511,8 +490,6 @@ ephy_shell_class_init (EphyShellClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
-  GtkApplicationClass *gtk_application_class = GTK_APPLICATION_CLASS (klass);
-  EphyEmbedShellClass *embed_shell_class = EPHY_EMBED_SHELL_CLASS (klass);
 
   object_class->dispose = ephy_shell_dispose;
   object_class->finalize = ephy_shell_finalize;
@@ -523,72 +500,9 @@ ephy_shell_class_init (EphyShellClass *klass)
   application_class->before_emit = ephy_shell_before_emit;
   application_class->add_platform_data = ephy_shell_add_platform_data;
 
-  gtk_application_class->window_added = ephy_shell_window_added;
-  gtk_application_class->window_removed = ephy_shell_window_removed;
-
-  embed_shell_class->get_embed_single = impl_get_embed_single;
-
   g_type_class_add_private (object_class, sizeof(EphyShellPrivate));
 }
 
-static EphyEmbed *
-ephy_shell_new_window_cb (EphyEmbedSingle *single,
-                          EphyEmbed *parent_embed,
-                          EphyWebViewChrome chromemask,
-                          EphyShell *shell)
-{
-  GtkWidget *parent = NULL;
-  gboolean is_popup;
-  EphyNewTabFlags flags = EPHY_NEW_TAB_DONT_SHOW_WINDOW |
-    EPHY_NEW_TAB_APPEND_LAST |
-    EPHY_NEW_TAB_IN_NEW_WINDOW |
-    EPHY_NEW_TAB_JUMP;
-
-  LOG ("ephy_shell_new_window_cb tab chrome %d", chromemask);
-
-  if (g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
-                              EPHY_PREFS_LOCKDOWN_JAVASCRIPT_CHROME))
-    chromemask = EPHY_WEB_VIEW_CHROME_ALL;
-
-  if (parent_embed != NULL) {
-    /* this will either be a EphyWindow, or the embed itself
-     * (in case it's about to be destroyed, which means it's already
-     * removed from its tab)
-     */
-    parent = gtk_widget_get_toplevel (GTK_WIDGET (parent_embed));
-  }
-
-  /* Any window opened with a toolbar is *not* a popup.
-   */
-  is_popup = (chromemask & EPHY_WEB_VIEW_CHROME_TOOLBAR) == 0;
-
-  return ephy_shell_new_tab_full
-    (shell,
-     EPHY_IS_WINDOW (parent) ? EPHY_WINDOW (parent) : NULL,
-     NULL, NULL, flags, chromemask, is_popup, 0);
-}
-
-static GObject*
-impl_get_embed_single (EphyEmbedShell *embed_shell)
-{
-  EphyShell *shell = EPHY_SHELL (embed_shell);
-  EphyShellPrivate *priv = shell->priv;
-  GObject *embed_single;
-
-  embed_single = EPHY_EMBED_SHELL_CLASS (ephy_shell_parent_class)->get_embed_single (embed_shell);
-
-  if (embed_single != NULL &&
-      priv->embed_single_connected == FALSE) {
-    g_signal_connect_object (embed_single, "new-window",
-                             G_CALLBACK (ephy_shell_new_window_cb),
-                             shell, G_CONNECT_AFTER);
-
-    priv->embed_single_connected = TRUE;
-  }
-
-  return embed_single;
-}
-
 #ifdef HAVE_WEBKIT2
 static void
 download_started_cb (WebKitWebContext *web_context,
@@ -659,7 +573,7 @@ ephy_shell_init (EphyShell *shell)
   /* Initialize the favicon cache as early as possible, or further
      calls to webkit_web_context_get_favicon_database will fail. */
   mode = ephy_embed_shell_get_mode (ephy_embed_shell_get_default ());
-  favicon_db_path = g_build_filename (mode == EPHY_EMBED_SHELL_MODE_PRIVATE ?
+  favicon_db_path = g_build_filename (EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (mode) ?
                                       ephy_dot_dir () : g_get_user_cache_dir (),
                                       g_get_prgname (), "icondatabase", NULL);
 
@@ -684,10 +598,9 @@ ephy_shell_dispose (GObject *object)
   g_clear_object (&priv->bookmarks);
   g_clear_object (&priv->network_monitor);
 
-  if (priv->windows != NULL) {
-    LOG ("Free browser window list");
-    g_list_free (priv->windows);
-    priv->windows = NULL;
+  if (priv->open_uris_idle_id > 0)  {
+    g_source_remove (priv->open_uris_idle_id);
+    priv->open_uris_idle_id = 0;
   }
 
   G_OBJECT_CLASS (ephy_shell_parent_class)->dispose (object);
@@ -754,15 +667,26 @@ ephy_shell_new_tab_full (EphyShell *shell,
   gboolean fullscreen_lockdown = FALSE;
   gboolean in_new_window = TRUE;
   gboolean open_page = FALSE;
+  gboolean delayed_open_page = FALSE;
   gboolean jump_to = FALSE;
   gboolean active_is_blank = FALSE;
   gboolean copy_history = TRUE;
   gboolean is_empty = FALSE;
   int position = -1;
 
-  embed_shell = ephy_embed_shell_get_default ();
+  g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
+  g_return_val_if_fail (EPHY_IS_WINDOW (parent_window) || !parent_window, NULL);
+  g_return_val_if_fail (EPHY_IS_EMBED (previous_embed) || !previous_embed, NULL);
+#ifdef HAVE_WEBKIT2
+  g_return_val_if_fail (WEBKIT_IS_URI_REQUEST (request) || !request, NULL);
+#else
+  g_return_val_if_fail (WEBKIT_IS_NETWORK_REQUEST (request) || !request, NULL);
+#endif
+
+  embed_shell = EPHY_EMBED_SHELL (shell);
 
   if (flags & EPHY_NEW_TAB_OPEN_PAGE) open_page = TRUE;
+  if (flags & EPHY_NEW_TAB_DELAYED_OPEN_PAGE) delayed_open_page = TRUE;
   if (flags & EPHY_NEW_TAB_IN_NEW_WINDOW) in_new_window = TRUE;
   if (flags & EPHY_NEW_TAB_IN_EXISTING_WINDOW) in_new_window = FALSE;
   if (flags & EPHY_NEW_TAB_DONT_COPY_HISTORY) copy_history = FALSE;
@@ -771,7 +695,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
   fullscreen_lockdown = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
                                                 EPHY_PREFS_LOCKDOWN_FULLSCREEN);
   in_new_window = in_new_window && !fullscreen_lockdown;
-  g_return_val_if_fail (open_page == (gboolean)(request != NULL), NULL);
+  g_return_val_if_fail ((open_page || delayed_open_page) == (gboolean)(request != NULL), NULL);
 
   LOG ("Opening new tab parent-window %p parent-embed %p in-new-window:%s jump-to:%s",
        parent_window, previous_embed, in_new_window ? "t" : "f", jump_to ? "t" : "f");
@@ -791,6 +715,9 @@ ephy_shell_new_tab_full (EphyShell *shell,
       g_warning ("Requested to append new tab after parent, but 'previous_embed' was NULL");
   }
 
+  if (flags & EPHY_NEW_TAB_FIRST)
+    position = 0;
+
   if (flags & EPHY_NEW_TAB_FROM_EXTERNAL) {
     /* If the active embed is blank, use that to open the url and jump to it */
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
@@ -834,7 +761,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
     ephy_window_activate_location (window);
     ephy_web_view_load_homepage (view);
     is_empty = TRUE;
-  } else if (flags & EPHY_NEW_TAB_OPEN_PAGE) {
+  } else if (open_page) {
     ephy_web_view_load_request (ephy_embed_get_web_view (embed),
                                 request);
 
@@ -843,7 +770,8 @@ ephy_shell_new_tab_full (EphyShell *shell,
 #else
     is_empty = ephy_embed_utils_url_is_empty (webkit_network_request_get_uri (request));
 #endif
-  }
+  } else if (delayed_open_page)
+      ephy_embed_set_delayed_load_request (embed, request);
 
   /* Make sure the initial focus is somewhere sensible and not, for
    * example, on the reload button.
@@ -913,7 +841,7 @@ ephy_shell_new_tab (EphyShell *shell,
  *
  * Return value: (transfer none): the current session.
  **/
-GObject *
+EphySession *
 ephy_shell_get_session (EphyShell *shell)
 {
   g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
@@ -921,7 +849,7 @@ ephy_shell_get_session (EphyShell *shell)
   if (shell->priv->session == NULL)
     shell->priv->session = g_object_new (EPHY_TYPE_SESSION, NULL);
 
-  return G_OBJECT (shell->priv->session);
+  return shell->priv->session;
 }
 
 /**
@@ -944,7 +872,7 @@ ephy_shell_get_bookmarks (EphyShell *shell)
  *
  * Return value: (transfer none):
  **/
-GObject *
+GNetworkMonitor *
 ephy_shell_get_net_monitor (EphyShell *shell)
 {
   EphyShellPrivate *priv = shell->priv;
@@ -952,7 +880,7 @@ ephy_shell_get_net_monitor (EphyShell *shell)
   if (priv->network_monitor == NULL)
     priv->network_monitor = g_network_monitor_get_default ();
 
-  return G_OBJECT (priv->network_monitor);
+  return priv->network_monitor;
 }
 
 /**
@@ -1073,20 +1001,15 @@ ephy_shell_set_startup_context (EphyShell *shell,
   shell->priv->startup_context = ctx;
 }
 
-GList *
-ephy_shell_get_windows (EphyShell *shell)
-{
-  g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
-
-  return g_list_copy (shell->priv->windows);
-}
-
 guint
 ephy_shell_get_n_windows (EphyShell *shell)
 {
+  GList *list;
+
   g_return_val_if_fail (EPHY_IS_SHELL (shell), 0);
 
-  return g_list_length (shell->priv->windows);
+  list = gtk_application_get_windows (GTK_APPLICATION (shell));
+  return g_list_length (list);
 }
 
 EphyWindow*
@@ -1101,7 +1024,7 @@ ephy_shell_get_main_window (EphyShell *shell)
   /* Select the window with most tabs in the current workspace as the window to
    * use for opening a new tab on, if that turns out to be the case.
    */
-  windows = ephy_shell_get_windows (shell);
+  windows = gtk_application_get_windows (GTK_APPLICATION (shell));
 
   for (iter = windows; iter != NULL; iter = iter->next) {
     EphyWindow *candidate = EPHY_WINDOW (iter->data);
@@ -1122,8 +1045,6 @@ ephy_shell_get_main_window (EphyShell *shell)
       window = candidate;
   }
 
-  g_list_free (windows);
-
   return window;
 }
 
@@ -1135,9 +1056,9 @@ ephy_shell_close_all_windows (EphyShell *shell)
 
   g_return_val_if_fail (EPHY_IS_SHELL (shell), FALSE);
 
-  ephy_session_close (EPHY_SESSION (ephy_shell_get_session (shell)));
+  ephy_session_close (ephy_shell_get_session (shell));
 
-  windows = shell->priv->windows;
+  windows = gtk_application_get_windows (GTK_APPLICATION (shell));
   while (windows) {
     EphyWindow *window = EPHY_WINDOW (windows->data);
 
@@ -1151,3 +1072,131 @@ ephy_shell_close_all_windows (EphyShell *shell)
 
   return retval;
 }
+
+typedef struct {
+  EphyShell *shell;
+  EphySession *session;
+  EphyWindow *window;
+  char **uris;
+  EphyNewTabFlags flags;
+  guint32 user_time;
+  guint current_uri;
+} OpenURIsData;
+
+static OpenURIsData *
+open_uris_data_new (EphyShell *shell,
+                    const char **uris,
+                    EphyStartupFlags startup_flags,
+                    guint32 user_time)
+{
+  OpenURIsData *data;
+  gboolean new_windows_in_tabs;
+  gboolean have_uris;
+
+  data = g_slice_new0 (OpenURIsData);
+  data->shell = shell;
+  data->session = g_object_ref (ephy_shell_get_session (shell));
+  data->uris = g_strdupv ((char **)uris);
+  data->user_time = user_time;
+
+  data->window = ephy_shell_get_main_window (shell);
+
+  new_windows_in_tabs = g_settings_get_boolean (EPHY_SETTINGS_MAIN,
+                                                EPHY_PREFS_NEW_WINDOWS_IN_TABS);
+
+  have_uris = ! (g_strv_length ((char **)uris) == 1 && g_str_equal (uris[0], ""));
+
+  if (startup_flags & EPHY_STARTUP_NEW_TAB)
+    data->flags |= EPHY_NEW_TAB_FROM_EXTERNAL;
+
+  if (startup_flags & EPHY_STARTUP_NEW_WINDOW) {
+    data->window = NULL;
+    data->flags |= EPHY_NEW_TAB_IN_NEW_WINDOW;
+  } else if (startup_flags & EPHY_STARTUP_NEW_TAB || (new_windows_in_tabs && have_uris)) {
+    data->flags |= EPHY_NEW_TAB_IN_EXISTING_WINDOW | EPHY_NEW_TAB_JUMP | EPHY_NEW_TAB_PRESENT_WINDOW;
+  } else if (!have_uris) {
+    data->window = NULL;
+    data->flags |= EPHY_NEW_TAB_IN_NEW_WINDOW;
+  }
+
+  g_application_hold (G_APPLICATION (shell));
+
+  return data;
+}
+
+static void
+open_uris_data_free (OpenURIsData *data)
+{
+  g_application_release (G_APPLICATION (data->shell));
+  g_object_unref (data->session);
+  g_strfreev (data->uris);
+
+  g_slice_free (OpenURIsData, data);
+}
+
+static gboolean
+ephy_shell_open_uris_idle (OpenURIsData *data)
+{
+  EphyEmbed *embed;
+  EphyNewTabFlags page_flags;
+  const char *url;
+#ifdef HAVE_WEBKIT2
+  WebKitURIRequest *request = NULL;
+#else
+  WebKitNetworkRequest *request = NULL;
+#endif
+
+  url = data->uris[data->current_uri];
+  if (url[0] == '\0') {
+    page_flags = EPHY_NEW_TAB_HOME_PAGE;
+  } else {
+    page_flags = EPHY_NEW_TAB_OPEN_PAGE;
+#ifdef HAVE_WEBKIT2
+    request = webkit_uri_request_new (url);
+#else
+    request = webkit_network_request_new (url);
+#endif
+  }
+
+  embed = ephy_shell_new_tab_full (ephy_shell_get_default (),
+                                   data->window,
+                                   NULL /* parent tab */,
+                                   request,
+                                   data->flags | page_flags,
+                                   EPHY_WEB_VIEW_CHROME_ALL,
+                                   FALSE /* is popup? */,
+                                   data->user_time);
+
+  if (request)
+    g_object_unref (request);
+
+  data->window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed)));
+  data->current_uri++;
+
+  return data->uris[data->current_uri] != NULL;
+}
+
+static void
+ephy_shell_open_uris_idle_done (OpenURIsData *data)
+{
+  data->shell->priv->open_uris_idle_id = 0;
+  open_uris_data_free (data);
+}
+
+void
+ephy_shell_open_uris (EphyShell *shell,
+                      const char **uris,
+                      EphyStartupFlags startup_flags,
+                      guint32 user_time)
+{
+  g_return_if_fail (EPHY_IS_SHELL (shell));
+
+  if (shell->priv->open_uris_idle_id == 0) {
+    shell->priv->open_uris_idle_id =
+      g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                       (GSourceFunc)ephy_shell_open_uris_idle,
+                       open_uris_data_new (shell, uris, startup_flags, user_time),
+                       (GDestroyNotify)ephy_shell_open_uris_idle_done);
+  }
+}
+
