@@ -81,6 +81,13 @@ window_cmd_file_print (GtkAction *action,
 	ephy_web_view_print (view);
 }
 
+void
+window_cmd_undo_close_tab (GtkAction *action,
+			   EphyWindow *window)
+{
+	ephy_session_undo_close_tab (ephy_shell_get_session (ephy_shell_get_default ()));
+}
+
 void
 window_cmd_file_send_to	(GtkAction *action,
 			 EphyWindow *window)
@@ -837,6 +844,7 @@ dialog_save_as_application_response_cb (GtkDialog *dialog,
 
 		notify_notification_set_timeout (notification, NOTIFY_EXPIRES_DEFAULT);
 		notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
+		notify_notification_set_hint (notification, "desktop-entry", g_variant_new_string ("epiphany"));
 		notify_notification_set_hint (notification, "transient", g_variant_new_boolean (TRUE));
 		notify_notification_show (notification, NULL);
 	}
@@ -1675,7 +1683,7 @@ window_cmd_help_about (GtkAction *action,
 			       "program-name", _("Web"),
 			       "version", VERSION,
 			       "copyright", "Copyright © 2002–2004 Marco Pesenti Gritti\n"
-			                    "Copyright © 2003–2012 The Web Developers",
+			                    "Copyright © 2003–2013 The Web Developers",
 			       "artists", artists,
 			       "authors", authors,
 			       "comments", comments,
